package com.engine.kq.cmd.report;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.kq.biz.*;
import com.engine.kq.util.ExcelUtil;
import com.engine.kq.util.KQDurationCalculatorUtil;
import com.engine.kq.util.UtilKQ;
import com.engine.xmgsecond.entity.KqFormatTotal;
import com.engine.xmgsecond.util.KqCalulateUtil;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.company.SubCompanyComInfo;
import weaver.hrm.job.JobTitlesComInfo;
import weaver.hrm.resource.ResourceComInfo;
import weaver.systeminfo.SystemEnv;
import com.google.common.collect.Lists;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class ExportExcelCmd extends AbstractCommonCommand<Map<String, Object>> {

  private HttpServletRequest request;
  private HttpServletResponse response;
  private List<String> lsFieldDataKey;

  private boolean isKqReportHalfOpen = "1".equals(new KQSettingsComInfo().getMain_val("kq_report_half"));

  public ExportExcelCmd(Map<String, Object> params, HttpServletRequest request, HttpServletResponse response, User user) {
    this.user = user;
    this.params = params;
    this.request = request;
    this.response = response;
    this.lsFieldDataKey = new ArrayList<>();
  }

  @Override
  public Map<String, Object> execute(CommandContext commandContext) {
    Map<String, Object> retmap = new HashMap<String, Object>();
    RecordSet rs = new RecordSet();
    String sql = "";
    try {
      SubCompanyComInfo subCompanyComInfo = new SubCompanyComInfo();
      DepartmentComInfo departmentComInfo = new DepartmentComInfo();
      ResourceComInfo resourceComInfo = new ResourceComInfo();
      JobTitlesComInfo jobTitlesComInfo = new JobTitlesComInfo();
      KQLeaveRulesBiz kqLeaveRulesBiz = new KQLeaveRulesBiz();
      KQSettingsComInfo kqSettingsComInfo = new KQSettingsComInfo();
      String leavetype_is_on = Util.null2String(kqSettingsComInfo.getMain_val("leavetype_is_on"),"0");

      KQReportBiz kqReportBiz = new KQReportBiz();
      new KQReportBiz().insertKqReportExportLog(params, user);
      JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
      String attendanceSerial = Util.null2String(jsonObj.get("attendanceSerial"));
      String fromDate = Util.null2String(jsonObj.get("fromDate"));
      String toDate = Util.null2String(jsonObj.get("toDate"));
      String typeselect = Util.null2String(jsonObj.get("typeselect"));
      if (typeselect.length() == 0) typeselect = "3";
      if (!typeselect.equals("") && !typeselect.equals("0") && !typeselect.equals("6")) {
        if (typeselect.equals("1")) {
          fromDate = TimeUtil.getCurrentDateString();
          toDate = TimeUtil.getCurrentDateString();
        } else {
          fromDate = TimeUtil.getDateByOption(typeselect, "0");
          toDate = TimeUtil.getDateByOption(typeselect, "1");
        }
      }
      //人员状态
      String status = Util.null2String(jsonObj.get("status"));
      String subCompanyId = Util.null2String(jsonObj.get("subCompanyId"));
      String departmentId = Util.null2String(jsonObj.get("departmentId"));
      String resourceId = Util.null2String(jsonObj.get("resourceId"));
      String allLevel = Util.null2String(jsonObj.get("allLevel"));
      String isNoAccount = Util.null2String(jsonObj.get("isNoAccount"));
      String viewScope = Util.null2String(jsonObj.get("viewScope"));
      List<String> showColumns = Util.splitString2List(Util.null2String(jsonObj.get("showColumns")), ",");
      showColumns.add("lastname");
      showColumns.removeIf(showColumn->showColumn.trim().equals(""));

      List<String> tmpShowColumns = new ArrayList<>();
      for(String showColumn:showColumns){
        tmpShowColumns.add(showColumn);
        String cascadekey = "";
        if(showColumn.equals("beLate")){
          cascadekey = "beLateMins";
        }else if(showColumn.equals("leaveEearly")){
          cascadekey = "leaveEarlyMins";
        }else if(showColumn.equals("graveBeLate")){
          cascadekey = "graveBeLateMins";
        }else if(showColumn.equals("graveLeaveEarly")){
          cascadekey = "graveLeaveEarlyMins";
        }else if(showColumn.equals("absenteeism")){
          cascadekey = "absenteeismMins";
        }else if(showColumn.equals("overtime")){
          tmpShowColumns.add("overtime_4leave");
          tmpShowColumns.add("overtime_nonleave");
          tmpShowColumns.add("workingDayOvertime_nonleave");
          tmpShowColumns.add("workingDayOvertime_4leave");
          tmpShowColumns.add("restDayOvertime_nonleave");
          tmpShowColumns.add("restDayOvertime_4leave");
          tmpShowColumns.add("holidayOvertime_4leave");
          tmpShowColumns.add("holidayOvertime_nonleave");
        }
        if(cascadekey.length()>0){
          tmpShowColumns.add(cascadekey);
        }
      }
      showColumns = tmpShowColumns;

      String rightSql = new KQReportBiz().getReportRight("1", "" + user.getUID(), "a");

      LinkedHashMap<String, Object> workbook = new LinkedHashMap<>();
      List<Object> lsSheet = new ArrayList<>();
      Map<String, Object> sheet = null;
      List<Object> titleList = new ArrayList<>();
      Map<String, Object> title = null;
      List<List<Object>> dataList = new ArrayList<>();
      List<Object> data = null;
      List<Map<String, Object>> constraintList = null;

      sheet = new HashMap<>();
      sheet.put("sheetName", SystemEnv.getHtmlLabelName(390351, user.getLanguage()));
      sheet.put("sheetTitle", SystemEnv.getHtmlLabelName(390351, user.getLanguage()));
      boolean isEnd = false;
      Calendar cal = DateUtil.getCalendar();

      List<Map<String, Object>> leaveRules = kqLeaveRulesBiz.getAllLeaveRules();
      Map<String, Object> mapChildColumnInfo = null;
      List<Object> childColumns = null;
      KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
      while (kqReportFieldComInfo.next()) {
        if (Util.null2String(kqReportFieldComInfo.getParentid()).length() > 0) continue;
        if(kqReportFieldComInfo.getFieldname().equals("kqCalendar"))continue;
        if(KQReportFieldComInfo.cascadekey2fieldname.keySet().contains(kqReportFieldComInfo.getFieldname()))continue;
        if (!kqReportFieldComInfo.getReportType().equals("all") && !kqReportFieldComInfo.getReportType().equals("month"))
          continue;
        if (!showColumns.contains(kqReportFieldComInfo.getFieldname())&&!showColumns.contains(kqReportFieldComInfo.getParentid())) continue;
        if("leave".equalsIgnoreCase(kqReportFieldComInfo.getFieldname())&&leaveRules.size()==0){
          continue;
        }
        title = new HashMap<>();
        String unitType = KQReportBiz.getUnitType(kqReportFieldComInfo, user);
        if(unitType.length()>0){
          title.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage())+ "(" + unitType + ")");
        }else{
          title.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()));
        }
        title.put("width", 30 * 256);
        this.lsFieldDataKey.add(kqReportFieldComInfo.getFieldname());
        mapChildColumnInfo = this.getChildColumnsInfo(kqReportFieldComInfo.getFieldname(), user);
        childColumns = (List<Object>) mapChildColumnInfo.get("childColumns");
        if (childColumns.size() > 0) {//跨列width取子列的width
          title.put("children", childColumns);
          title.put("colSpan", childColumns.size());
        } else {
          title.put("rowSpan", 3);
        }
        titleList.add(title);

        titleList.addAll(this.getCascadeKeyColumnsInfo(kqReportFieldComInfo.getCascadekey(),user));
      }

      String today = DateUtil.getCurrentDate();
//      if (DateUtil.compDate(today, toDate) > 0) {//结束如期不大于今天
//        toDate = today;
//      }

      if(showColumns.contains("kqCalendar")) {
        childColumns = new ArrayList<>();
        for (String date = fromDate; !isEnd; ) {
          if (date.equals(toDate)) isEnd = true;
          title = new HashMap<>();
          title.put("title", UtilKQ.getWeekDayShort(DateUtil.getWeek(date)-1,user.getLanguage()) +"\r\n"+ DateUtil.geDayOfMonth(date));
          title.put("width", 30 * 256);
          childColumns.add(title);
          cal.setTime(DateUtil.parseToDate(date));
          date = DateUtil.getDate(cal.getTime(), 1);
        }

        title = new HashMap();
        title.put("title", SystemEnv.getHtmlLabelName(386476, user.getLanguage()));
        if (childColumns.size() > 0) {//跨列width取子列的width
          title.put("children", childColumns);
          title.put("colSpan", childColumns.size());
        }
        titleList.add(title);
      }
      sheet.put("titleList", titleList);

      // 没有权限的人最大只支持查询本季
      Map<String, String> getRealDate = new KQReportBiz().realDate(fromDate, toDate, user, "1");
      fromDate = getRealDate.get("fromDate");
      toDate = getRealDate.get("toDate");
      if("-1".equals(fromDate) || "-1".equals(toDate)) {
        retmap.put("status", "1");
        return retmap;
      }

      List<List> paramLists = new ArrayList<>();
      List<Object> paramList = null;
      String forgotBeginWorkCheck_field = " sum(b.forgotBeginWorkCheck) ";

      if(rs.getDBType().equalsIgnoreCase("oracle")) {
        forgotBeginWorkCheck_field = " sum(nvl(b.forgotBeginWorkCheck,0))  ";
      }else if((rs.getDBType()).equalsIgnoreCase("mysql")){
        forgotBeginWorkCheck_field = " sum(ifnull(b.forgotBeginWorkCheck,0)) ";
      }else {
        forgotBeginWorkCheck_field = " sum(isnull(b.forgotBeginWorkCheck,0)) ";
      }

      Map<String,Object> definedFieldInfo = new KQFormatBiz().getDefinedField();
      String definedFieldSum = Util.null2String(definedFieldInfo.get("definedFieldSum"));
      String backFields = " a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1 as subcompanyid,a.departmentid,a.jobtitle," +
              " sum(b.workdays) as workdays,sum(b.workMins) as workMins,sum(b.attendancedays) as attendancedays," +
              " sum(b.attendanceMins) as attendanceMins,sum(b.beLate) as beLate,sum(b.beLateMins) as beLateMins, " +
              " sum(b.graveBeLate) as graveBeLate, sum(b.graveBeLateMins) as graveBeLateMins,sum(b.leaveEearly) as leaveEearly," +
              " sum(b.leaveEarlyMins) as leaveEarlyMins, sum(b.graveLeaveEarly) as graveLeaveEarly, " +
              " sum(b.graveLeaveEarlyMins) as graveLeaveEarlyMins,sum(b.absenteeism) as absenteeism, " +
              " sum(b.signdays) as signdays,sum(b.signmins) as signmins, "+
              " sum(b.absenteeismMins) as absenteeismMins, sum(b.forgotCheck)+"+forgotBeginWorkCheck_field+" as forgotCheck "+(definedFieldSum.length()>0?","+definedFieldSum+"":"");
      if(rs.getDBType().equals("oracle")){
        backFields = 	"/*+ index(kq_format_total IDX_KQ_FORMAT_TOTAL_KQDATE) */ "+backFields;
      }
      String sqlFrom = " from hrmresource a, kq_format_total b where a.id= b.resourceid and b.kqdate >='" + fromDate + "' and b.kqdate <='" + toDate + "'";
      String sqlWhere = rightSql;
      String groupBy = " group by a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1,a.departmentid,a.jobtitle ";
      if (subCompanyId.length() > 0) {
        sqlWhere +=" and a.subcompanyid1 in("+KQReportBiz.splitParamStr(subCompanyId)+") ";
        for(String bean : subCompanyId.split(",")) {
          paramList = new ArrayList<>();
          paramList.add(bean);
          paramLists.add(paramList);
        }
      }

      if (departmentId.length() > 0) {
        sqlWhere +=" and a.departmentid in("+KQReportBiz.splitParamStr(departmentId)+") ";
        for(String bean : departmentId.split(",")) {
          paramList = new ArrayList<>();
          paramList.add(bean);
          paramLists.add(paramList);
        }
      }

      if (resourceId.length() > 0) {
        sqlWhere +=" and a.id in("+KQReportBiz.splitParamStr(resourceId)+") ";
        for(String bean : resourceId.split(",")) {
          paramList = new ArrayList<>();
          paramList.add(bean);
          paramLists.add(paramList);
        }
      }

      if (viewScope.equals("4")) {//我的下属
        if (allLevel.equals("1")) {//所有下属
          sqlWhere += " and a.managerstr like '%," + user.getUID() + ",%'";
        } else {
          sqlWhere += " and a.managerid=" + user.getUID();//直接下属
        }
      }
      if (!"1".equals(isNoAccount)) {
        sqlWhere += " and a.loginid is not null " + (rs.getDBType().equals("oracle") ? "" : " and a.loginid<>'' ");
      }

      if(status.length()>0){
        if (!status.equals("8") && !status.equals("9")) {
          sqlWhere += " and a.status = ?";
          paramList = new ArrayList<>();
          paramList.add(status);
          paramLists.add(paramList);
        }else if (status.equals("8")) {
          sqlWhere += " and (a.status = 0 or a.status = 1 or a.status = 2 or a.status = 3) ";
        }
      }

      String orderBy = " order by a.dsporder asc, a.lastname asc ";
      String descOrderBy = " order by a.dsporder desc, a.lastname desc ";
      sql = "select " + backFields + sqlFrom + sqlWhere + groupBy + orderBy;

      //System.out.println("start" + DateUtil.getFullDate());


      String show_card_source = Util.null2String(kqSettingsComInfo.getMain_val("show_card_source"),"0");//是否显示打卡数据，以及打卡数据来源
      params.put("show_card_source",show_card_source);
      KQOvertimeRulesBiz kqOvertimeRulesBiz = new KQOvertimeRulesBiz();
      int uintType = kqOvertimeRulesBiz.getMinimumUnit();//当前加班单位
      double hoursToDay = kqOvertimeRulesBiz.getHoursToDay();//当前天跟小时计算关系
      params.put("uintType",uintType);
      params.put("hoursToDay",hoursToDay);
      Map<String, Object> flowData = new KQReportBiz().getFlowData(params, user);
      //System.out.println("end" + DateUtil.getFullDate());

      // #1475814-概述：满足考勤报分部部门显示及导出时显示全路径
      String fullPathMainKey = "show_full_path";
      String isShowFullPath = Util.null2String(kqSettingsComInfo.getMain_val(fullPathMainKey),"0");

      rs.executeQuery(sql, paramLists);
      int calIndex = -1;
      while (rs.next()) {
        data = new ArrayList<>();
        String id = rs.getString("id");
        for (int fieldDataKeyIdx =0;fieldDataKeyIdx<lsFieldDataKey.size();fieldDataKeyIdx++) {
          String fieldName = lsFieldDataKey.get(fieldDataKeyIdx);
          String fieldid =  KQReportFieldComInfo.field2Id.get(fieldName);
          String fieldValue = "";
          if (fieldName.equals("subcompany")) {
            String tmpSubcompanyId = Util.null2String(rs.getString("subcompanyid"));
            if (tmpSubcompanyId.length() == 0) {
              tmpSubcompanyId = Util.null2String(resourceComInfo.getSubCompanyID(id));
            }

            fieldValue = "1".equals(isShowFullPath) ?
                    SubCompanyComInfo.getSubcompanyRealPath(tmpSubcompanyId, "/", "0") :
                    subCompanyComInfo.getSubCompanyname(tmpSubcompanyId);

            // fieldValue = subCompanyComInfo.getSubCompanyname(tmpSubcompanyId);
          } else if (fieldName.equals("department")) {
            String tmpDepartmentId = Util.null2String(rs.getString("departmentid"));
            if (tmpDepartmentId.length() == 0) {
              tmpDepartmentId = Util.null2String(resourceComInfo.getDepartmentID(id));
            }

            fieldValue = "1".equals(isShowFullPath) ?
                    departmentComInfo.getDepartmentRealPath(tmpDepartmentId, "/", "0") :
                    departmentComInfo.getDepartmentname(tmpDepartmentId);

            // fieldValue = departmentComInfo.getDepartmentname(tmpDepartmentId);
          } else if(fieldName.equals("zcgzts")){
            String rlzqqxsTime = getXcqts(fromDate,id);
            fieldValue = rlzqqxsTime;
          } else if (fieldName.equals("jobtitle")) {
            String tmpJobtitleId = Util.null2String(rs.getString("jobtitle"));
            if (tmpJobtitleId.length() == 0) {
              tmpJobtitleId = Util.null2String(resourceComInfo.getJobTitle(id));
            }
            fieldValue = jobTitlesComInfo.getJobTitlesname(tmpJobtitleId);
          } else if("kqGroup".equals(fieldName)){
            //QC3685035 考勤月度报表增加所属所属考勤组
            KQGroupComInfo kqGroupComInfo = new KQGroupComInfo();
            KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
            String groupId = kqGroupMemberComInfo.getKQGroupId(id, toDate);
            fieldValue = kqGroupComInfo.getGroupname(groupId);
//						fieldValue = kqGroupComInfo.getGroupname(Util.null2String(rs.getString("groupid")));
          }else if("absenteeismDays".equals(fieldName)){
            //QC3685035 考勤月度报表增加缺勤天数
            List<KqFormatTotal> kqFormatTotalList = KqCalulateUtil.abnormalAttendance(id, fromDate, toDate);
            fieldValue = String.format("%.1f", kqFormatTotalList.stream().mapToDouble(KqFormatTotal::getAbsenceDays).sum());
          }else if("nigntWork".equals(fieldName)){
            //QC3685035 考勤月度报表增加夜班次数
            fieldValue = String.valueOf(KqCalulateUtil.nightWorkSums(id,fromDate,toDate));
          }else if (fieldName.equals("attendanceSerial")) {
            List<String> serialIds = null;
            if (Util.null2String(jsonObj.get("attendanceSerial")).length() > 0) {
              serialIds = Util.splitString2List(Util.null2String(jsonObj.get("attendanceSerial")), ",");
              for(int i=0;serialIds!=null&&i<serialIds.size();i++){
                data.add(kqReportBiz.getSerialCount(id,fromDate,toDate,serialIds.get(i)));
              }
            }else{
              data.add("");
            }
            continue;
          } else if (fieldName.equals("leave")) {//请假
            List<Map<String, Object>> allLeaveRules = kqLeaveRulesBiz.getAllLeaveRules();
            if (("1").equals(leavetype_is_on)) {
              KQLeaveRulesComInfo kqLeaveRulesComInfo = new KQLeaveRulesComInfo();
              for (int i = allLeaveRules.size() - 1; i >= 0; i--) {
                Map<String, Object> leaveRule = allLeaveRules.get(i);
                String leaveId = Util.null2String(leaveRule.get("id"));
                boolean flag = kqLeaveRulesBiz.filterSubLeaveRule(kqLeaveRulesComInfo, user, leaveId);
                if (flag) {
                  allLeaveRules.remove(i);
                }
              }
            }
            Map<String, Object> leaveRule = null;
            for (int i = 0; allLeaveRules != null && i < allLeaveRules.size(); i++) {
              leaveRule = allLeaveRules.get(i);
              String flowType = Util.null2String("leaveType_" + leaveRule.get("id"));
              String leaveData = Util.null2String(flowData.get(id + "|" + flowType));
              String flowLeaveBackType = Util.null2String("leavebackType_" + leaveRule.get("id"));
              String leavebackData = Util.null2s(Util.null2String(flowData.get(id + "|" + flowLeaveBackType)), "0.0");
              String b_flowLeaveData = "";
              String flowLeaveData = "";
              try {
                //以防止出现精度问题
                if (leaveData.length() == 0) {
                  leaveData = "0.0";
                }
                if (leavebackData.length() == 0) {
                  leavebackData = "0.0";
                }
                BigDecimal b_leaveData = new BigDecimal(leaveData);
                BigDecimal b_leavebackData = new BigDecimal(leavebackData);
                b_flowLeaveData = b_leaveData.subtract(b_leavebackData).toString();
                if(Util.getDoubleValue(b_flowLeaveData, -1) < 0){
                  b_flowLeaveData = "0.0";
                }
              } catch (Exception e) {
                writeLog("GetKQReportCmd:leaveData" + leaveData + ":leavebackData:" + leavebackData + ":" + e);
              }

              if (b_flowLeaveData.length() > 0) {
                flowLeaveData = KQDurationCalculatorUtil.getDurationRound(b_flowLeaveData);
              } else {
                flowLeaveData = KQDurationCalculatorUtil.getDurationRound(Util.null2String(Util.getDoubleValue(leaveData, 0.0) - Util.getDoubleValue(leavebackData, 0.0)));
              }
              data.add(flowLeaveData);
            }
            continue;
          }else if(fieldName.equals("overtime")){
            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|workingDayOvertime_nonleave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("workingDayOvertime_nonleave"))));

            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|restDayOvertime_nonleave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("restDayOvertime_nonleave"))));

            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|holidayOvertime_nonleave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("holidayOvertime_nonleave"))));

            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|workingDayOvertime_4leave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("workingDayOvertime_4leave"))));

            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|restDayOvertime_4leave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("restDayOvertime_4leave"))));

            fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id + "|holidayOvertime_4leave")));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("holidayOvertime_4leave"))));

            double workingDayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|workingDayOvertime_4leave")));
            workingDayOvertime_4leave = workingDayOvertime_4leave<0?0:workingDayOvertime_4leave;
            double restDayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|restDayOvertime_4leave")));
            restDayOvertime_4leave = restDayOvertime_4leave<0?0:restDayOvertime_4leave;
            double holidayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|holidayOvertime_4leave")));
            holidayOvertime_4leave = holidayOvertime_4leave<0?0:holidayOvertime_4leave;

            double workingDayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|workingDayOvertime_nonleave")));
            workingDayOvertime_nonleave = workingDayOvertime_nonleave<0?0:workingDayOvertime_nonleave;
            double restDayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|restDayOvertime_nonleave")));
            restDayOvertime_nonleave = restDayOvertime_nonleave<0?0:restDayOvertime_nonleave;
            double holidayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|holidayOvertime_nonleave")));
            holidayOvertime_nonleave = holidayOvertime_nonleave<0?0:holidayOvertime_nonleave;
            fieldValue = KQDurationCalculatorUtil.getDurationRound(String.valueOf(workingDayOvertime_4leave+restDayOvertime_4leave+holidayOvertime_4leave+
                    workingDayOvertime_nonleave+restDayOvertime_nonleave+holidayOvertime_nonleave));
            data.add(getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(KQReportFieldComInfo.field2Id.get("overtimeTotal"))));
            continue;
          }else if(fieldName.equals("businessLeave") || fieldName.equals("officialBusiness")){
            String businessLeaveData = Util.null2s(Util.null2String(flowData.get(id+"|"+fieldName)),"0.0");
            String backType = fieldName+"_back";
            String businessLeavebackData = Util.null2s(Util.null2String(flowData.get(id+"|"+backType)),"0.0");
            String businessLeave = "";
            try{
              //以防止出现精度问题
              if(businessLeaveData.length() == 0){
                businessLeaveData = "0.0";
              }
              if(businessLeavebackData.length() == 0){
                businessLeavebackData = "0.0";
              }
              BigDecimal b_businessLeaveData = new BigDecimal(businessLeaveData);
              BigDecimal b_businessLeavebackData = new BigDecimal(businessLeavebackData);
              businessLeave = b_businessLeaveData.subtract(b_businessLeavebackData).toString();
              if(Util.getDoubleValue(businessLeave, -1) < 0){
                businessLeave = "0.0";
              }
            }catch (Exception e){
            }
            fieldValue = KQDurationCalculatorUtil.getDurationRound(businessLeave);
          } else if(Util.null2String(kqReportFieldComInfo.getCascadekey(fieldid)).length()>0){
            fieldValue = Util.formatMultiLang(Util.null2String(rs.getString(fieldName)),""+user.getLanguage());
            data.add(fieldValue);

            List<String> lsCascadekey = Util.splitString2List(kqReportFieldComInfo.getCascadekey(fieldid),",");
            for(int i=0;i<lsCascadekey.size();i++){
              if(Util.null2String(rs.getString(lsCascadekey.get(i))).length()>0){
                fieldid =  KQReportFieldComInfo.field2Id.get(lsCascadekey.get(i));
                fieldValue = getFieldValueByUnitType(rs.getString(lsCascadekey.get(i)),kqReportFieldComInfo.getUnittype(fieldid));
              }else{
                fieldValue = "0";
              }
              data.add(fieldValue);
            }
            continue;
          }else {
            fieldValue = Util.formatMultiLang(Util.null2String(rs.getString(fieldName)),""+user.getLanguage());
            fieldValue = getFieldValueByUnitType(fieldValue,kqReportFieldComInfo.getUnittype(fieldid));
          }
          fieldValue = Util.formatMultiLang(fieldValue,""+user.getLanguage());
          data.add(fieldValue);
        }

        List<Object> amData = Lists.newArrayList();
        List<Object> pmData = Lists.newArrayList();
        if(isKqReportHalfOpen) {
          amData.addAll(data);
          pmData.addAll(data);
          calIndex = data.size();
        }

        if(showColumns.contains("kqCalendar")) {
//          Map<String, Object> detialDatas = kqReportBiz.getDetialDatas(id, fromDate, toDate, user,flowData,true);
          Map<String, Object> detialDatas = kqReportBiz.getDetialDatas(id, fromDate, toDate, user,flowData,true,uintType,show_card_source);
          String user_endDate = Util.null2String(resourceComInfo.getEndDate(id));
          String user_status = Util.null2String(resourceComInfo.getStatus(id));
          String begindate = Util.null2String(resourceComInfo.getCreatedate(id)).trim();
          String companyStartDate = Util.null2String(resourceComInfo.getCompanyStartDate(id)).trim();
          if(companyStartDate.length()!=10){
            companyStartDate = "";
          }
          if(companyStartDate.length()>0 && companyStartDate.indexOf("-")>0){
            begindate=companyStartDate;
          }
          isEnd = false;
          for (String date = fromDate; !isEnd; ) {
            if (date.equals(toDate)) isEnd = true;
            if(DateUtil.compDate(today, date)>0){
              if(isKqReportHalfOpen) {
                amData.add("");
                pmData.add("");
              } else {
                data.add("");
              }
            }else{
              if(!(user_status.equals("0")||user_status.equals("1")||user_status.equals("2")||user_status.equals("3"))){
                //其他状态
                if(user_endDate.length()>0  && date.compareTo(user_endDate)>0){//人员合同结束日期之后
                  if(isKqReportHalfOpen) {
                    amData.add("");
                    pmData.add("");
                  } else {
                    data.add("");
                  }
                  cal.setTime(DateUtil.parseToDate(date));
                  date = DateUtil.getDate(cal.getTime(), 1);
                  continue;
                }
              }
              if(begindate.length()>0 && date.compareTo(begindate)<0 ){//人员入职日期前无需计算考勤，如果没有入职日期，已创建日期为准
                if(isKqReportHalfOpen) {
                  amData.add("");
                  pmData.add("");
                } else {
                  data.add("");
                }
                cal.setTime(DateUtil.parseToDate(date));
                date = DateUtil.getDate(cal.getTime(), 1);
                continue;
              }
              if (detialDatas.get(id + "|" + date) != null) {
                if(isKqReportHalfOpen) {
                  amData.add(((Map<String, Object>) detialDatas.get(id + "|" + date)).get("textAM"));
                  pmData.add(((Map<String, Object>) detialDatas.get(id + "|" + date)).get("textPM"));
                } else {
                  data.add(((Map<String, Object>) detialDatas.get(id + "|" + date)).get("text"));
                }
              } else {
                if(isKqReportHalfOpen) {
                  amData.add(SystemEnv.getHtmlLabelName(26593, user.getLanguage()));
                  pmData.add(SystemEnv.getHtmlLabelName(26593, user.getLanguage()));
                } else {
                  data.add(SystemEnv.getHtmlLabelName(26593, user.getLanguage()));
                }
              }
            }
            cal.setTime(DateUtil.parseToDate(date));
            date = DateUtil.getDate(cal.getTime(), 1);
          }
        }
        if(isKqReportHalfOpen) {
          dataList.add(amData);
          dataList.add(pmData);
        } else {
          dataList.add(data);
        }
      }
      sheet.put("dataList", dataList);
      if(isKqReportHalfOpen) {
        sheet.put("calIndex", calIndex);
      }
      sheet.put("constraintList", constraintList);
      sheet.put("createFile", "1");
      lsSheet.add(sheet);

      workbook.put("sheet", lsSheet);
      String fileName = SystemEnv.getHtmlLabelName(390351, user.getLanguage())+" "+fromDate+" "+toDate;
      workbook.put("fileName", fileName);
      ExcelUtil ExcelUtil = new ExcelUtil();
      Map<String, Object> exportMap = ExcelUtil.export(workbook, request, response,true);
      retmap.putAll(exportMap);
      retmap.put("status", "1");
    } catch (Exception e) {
      retmap.put("status", "-1");
      retmap.put("message", SystemEnv.getHtmlLabelName(382661, user.getLanguage()));
      writeLog(e);
    }
    return retmap;
  }

  private Map<String, Object> getChildColumnsInfo(String parentid, User user) {
    Map<String, Object> returnMap = new HashMap<>();
    List<Object> titleList = new ArrayList<>();
    Map<String, Object> title = null;
    KQSettingsComInfo kqSettingsComInfo = new KQSettingsComInfo();
    String leavetype_is_on = Util.null2String(kqSettingsComInfo.getMain_val("leavetype_is_on"),"0");
    if (parentid.equals("attendanceSerial")) {//考勤班次
      KQShiftManagementComInfo kqShiftManagementComInfo = new KQShiftManagementComInfo();
      JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
      List<String> serialIds = null;
      if (Util.null2String(jsonObj.get("attendanceSerial")).length() > 0) {
        serialIds = Util.splitString2List(Util.null2String(jsonObj.get("attendanceSerial")), ",");
      }
      for (int i = 0; serialIds != null && i < serialIds.size(); i++) {
        title = new HashMap<>();
        title.put("title", Util.formatMultiLang(kqShiftManagementComInfo.getSerial(serialIds.get(i)),""+user.getLanguage()));
        title.put("width", 30 * 256);
        titleList.add(title);
      }
    } else if (parentid.equals("leave")) {
      KQLeaveRulesBiz kqLeaveRulesBiz = new KQLeaveRulesBiz();
      List<Map<String, Object>> leaveRules = kqLeaveRulesBiz.getAllLeaveRules();
      if (("1").equals(leavetype_is_on)) {
        KQLeaveRulesComInfo kqLeaveRulesComInfo = new KQLeaveRulesComInfo();
        for (int i = leaveRules.size() - 1; i >= 0; i--) {
          Map<String, Object> leaveRule = leaveRules.get(i);
          String leaveId = Util.null2String(leaveRule.get("id"));
          boolean flag = kqLeaveRulesBiz.filterSubLeaveRule(kqLeaveRulesComInfo, user, leaveId);
          if (flag) {
            leaveRules.remove(i);
          }
        }
      }
      for (int i = 0; leaveRules != null && i < leaveRules.size(); i++) {
        Map<String, Object> leaveRule = leaveRules.get(i);
        String name = Util.formatMultiLang(Util.null2String(leaveRule.get("name")),""+user.getLanguage());
        String unitType = Util.null2String(leaveRule.get("unitType"));
        String unitName = (KQUnitBiz.isLeaveHour(unitType)) ? SystemEnv.getHtmlLabelName(391, user.getLanguage()) : SystemEnv.getHtmlLabelName(1925, user.getLanguage());
        title = new HashMap<>();
        title.put("title", name + "(" + unitName + ")");
        title.put("width", 30 * 256);
        titleList.add(title);
      }
    }else if(parentid.equals("overtime")){
      String[] overtimeChild = {"overtime_nonleave","overtime_4leave","overtimeTotal"};
      for(int i=0;i<overtimeChild.length;i++){
        String id = overtimeChild[i];
        title = new HashMap();
        String fieldlabel = "";
        if("overtime_nonleave".equalsIgnoreCase(id)){
          fieldlabel = "125805";
          title.put("title", SystemEnv.getHtmlLabelNames(fieldlabel, user.getLanguage()));
          title.put("rowSpan","2");
        }else if("overtime_4leave".equalsIgnoreCase(id)){
          fieldlabel = "125804";
          title.put("title", SystemEnv.getHtmlLabelNames(fieldlabel, user.getLanguage()));
          title.put("rowSpan","2");
        }else{
          fieldlabel = "523";
          title.put("showDetial","1");
          String unitType = (KQOvertimeRulesBiz.getMinimumUnit()==3 || KQOvertimeRulesBiz.getMinimumUnit()==5 ||KQOvertimeRulesBiz.getMinimumUnit()==6)?"2":"1";
          String unitTypeName = "";
          if(Util.null2String(unitType).length()>0){
            if(unitType.equals("1")){
              unitTypeName=SystemEnv.getHtmlLabelName(1925, user.getLanguage());
            }else if(unitType.equals("2")){
              unitTypeName=SystemEnv.getHtmlLabelName(391, user.getLanguage());
            }else if(unitType.equals("3")){
              unitTypeName=SystemEnv.getHtmlLabelName(18083, user.getLanguage());
            }
          }
          title.put("title", SystemEnv.getHtmlLabelNames(fieldlabel, user.getLanguage())+ "(" + unitTypeName + ")");
        }

        Map<String,Object> mapChildColumnInfo = getChildColumnsInfo(id, user);
        int childWidth = 65;
        List<Object> childColumns = (List<Object>)mapChildColumnInfo.get("childColumns");
        if(childColumns.size()>0) {//跨列width取子列的width
          title.put("children", childColumns);
          childWidth = Util.getIntValue(Util.null2String(mapChildColumnInfo.get("sumChildColumnWidth")),65);
        }
        title.put("width", childWidth+"");
        titleList.add(title);
      }
    } else {
      KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
      while (kqReportFieldComInfo.next()) {
        if (kqReportFieldComInfo.getParentid().equals(parentid)) {
          if(!kqReportFieldComInfo.getReportType().equals("month"))continue;
          title = new HashMap<>();
          title.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()) + "(" + KQReportBiz.getUnitType(kqReportFieldComInfo, user) + ")");
          title.put("width", 30 * 256);
          titleList.add(title);
        }
      }
    }
    returnMap.put("childColumns", titleList);
    return returnMap;
  }

  private List<Object>  getCascadeKeyColumnsInfo(String cascadeKey, User user){
    List<Object> titleList = new ArrayList<>();
    Map<String, Object> title = null;
    if(Util.null2String(cascadeKey).length()==0){
      return titleList;
    }
    List<String> lsCascadeKey = Util.splitString2List(cascadeKey,",");
    KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
    for(int i=0;i<lsCascadeKey.size();i++){
      kqReportFieldComInfo.setTofirstRow();
      while (kqReportFieldComInfo.next()) {
        if(!kqReportFieldComInfo.getReportType().equals("month"))continue;
        if (kqReportFieldComInfo.getFieldname().equals(lsCascadeKey.get(i))){
          title = new HashMap<>();
          title.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()) + "(" + KQReportBiz.getUnitType(kqReportFieldComInfo, user) + ")");
          title.put("width", 30 * 256);

          titleList.add(title);
        }
      }
    }
    return titleList;
  }

  private String getFieldValueByUnitType(String fieldValue,String unittype){
    if (Util.null2String(unittype).length() > 0) {
      if (fieldValue.length() == 0) {
        fieldValue = "0";
      } else {
        if (unittype.equals("2")) {
          fieldValue = KQDurationCalculatorUtil.getDurationRound(("" + (Util.getDoubleValue(fieldValue) / 60.0)));
        }
      }
    }
    return fieldValue;
  }
  @Override
  public BizLogContext getLogContext() {
    return null;
  }

  /**
   * 找到人员入职日期
   *
   * @return
   */
  public static String findRzrq(String jbr) {
    RecordSet rs = new RecordSet();
    String sql = "select id,companystartdate from hrmresource where id ="+jbr;
    rs.executeQuery(sql);
    String companystartdate = "";
    if (rs.next()) {
      companystartdate = Util.null2String(rs.getString("companystartdate"));
    }
    return companystartdate;
  }


  /**
   * 找到人员离职日期
   *
   * @return
   */
  public static String findLzrq(String jbr) {
    RecordSet rs = new RecordSet();
    String sql = "select changedate from HrmStatusHistory where RESOURCEID ="+jbr+" and type_n = 5";
    rs.executeQuery(sql);
    String changedate = "";
    if (rs.next()) {
      changedate = Util.null2String(rs.getString("changedate"));
    }
    return changedate;
  }

  /**
   * 获取入离职缺勤小时
   * @param dateOri
   * @param userId
   * @return
   */
  public String getXcqts(String dateOri,String userId){
    //需出勤天数
    Integer xcqtsAll = 0;
    //获取员工的入职日期
    String rzrq = findRzrq(userId);
    //获取员工的离职日期
    String lzrq = findLzrq(userId);
    String dateOriSub = dateOri.substring(0,7);
    //本月第一天
    String fromDate = TimeUtil.getMonthBeginDay(dateOri);
    //本月最后一天
    String toDate = TimeUtil.getMonthEndDay(dateOri);
    //入职日期处理
    if(StringUtils.isNotBlank(rzrq)){
      String rzrqSub = rzrq.substring(0,7);
      if(rzrqSub.equals(dateOriSub)){
        //入职日期在当月
        Set<String> all = new HashSet<>();
        all.add(fromDate);
        List<String> back = days(fromDate,rzrq);
        all.addAll(back);
        for (String date : all) {
          Integer zcgzts = 0;
          KQHolidaySetComInfo holidaySetComInfo = new KQHolidaySetComInfo();
          String changeTypeNew = holidaySetComInfo.getChangeType("1", date);
//					if("2".equals(changeTypeNew)){
//						//调配工作日计正常工作天数
//						zcgzts = 1;
//					}
          if(DateUtil.getWeek(date) != 6 && DateUtil.getWeek(date) != 7){
            //不为周六周日
            if(!"3".equals(changeTypeNew)){
              //不为休息日
              zcgzts = 1;
            }
          }

          xcqtsAll = xcqtsAll+zcgzts;
        }

      }
    }
    //离职日期处理
    if(StringUtils.isNotBlank(lzrq)){
      String lzrqSub = lzrq.substring(0,7);
      if(lzrqSub.equals(dateOriSub)){
        //离职日期在当月
        Set<String> all = new HashSet<>();
        List<String> back = days(lzrq,toDate);
        all.addAll(back);
        all.add(toDate);
        for (String date : all) {
          Integer zcgzts = 0;
          KQHolidaySetComInfo holidaySetComInfo = new KQHolidaySetComInfo();
          String changeTypeNew = holidaySetComInfo.getChangeType("1", date);
//					if("2".equals(changeTypeNew)){
//						//调配工作日计正常工作天数
//						zcgzts = 1;
//					}
          if(DateUtil.getWeek(date) != 6 && DateUtil.getWeek(date) != 7){
            //不为周六周日
            if(!"3".equals(changeTypeNew)){
              //不为休息日
              zcgzts = 1;
            }
          }

          xcqtsAll = xcqtsAll+zcgzts;
        }

      }
    }
    Integer newXcqTime = xcqtsAll * 8;
    new BaseBean().writeLog(userId+"newXcqTime"+newXcqTime);
    return String.valueOf(newXcqTime);
  }

  /**
   * 获取两个日期之间所有的日期
   * @param date1
   * @param date2
   * @return
   */
  public static ArrayList days(String date1, String date2) {
    String dateFormat = "yyyy-MM-dd";
    SimpleDateFormat format = new SimpleDateFormat(dateFormat);
    ArrayList L = new ArrayList();
    if (date1.equals(date2)) {
      System.out.println("两个日期相等!");
      return L;
    }

    String tmp;
    if (date1.compareTo(date2) > 0) { // 确保 date1的日期不晚于date2
      tmp = date1;
      date1 = date2;
      date2 = tmp;
    }

    tmp = format.format(str2Date(date1).getTime() + 3600 * 24 * 1000);

    int num = 0;
    while (tmp.compareTo(date2) < 0) {
      L.add(tmp);
      num++;
      tmp = format.format(str2Date(tmp).getTime() + 3600 * 24 * 1000);
    }

    if (num == 0)
      System.out.println("两个日期相邻!");
    return L;
  }

  private static Date str2Date(String str) {
    String dateFormat = "yyyy-MM-dd";
    SimpleDateFormat format = new SimpleDateFormat(dateFormat);
    if (str == null)
      return null;

    try {
      return format.parse(str);
    } catch (ParseException e) {
      e.printStackTrace();
    }
    return null;
  }
}
